-- RS_DKP_Broadcasting.lua
-- Part of RS_DKP by Tim Morgan
-- Functions for the synchronization and cross-communication of multiple
-- simultaneous instances of RS_DKP.

--
-- CONSTANTS
--

local RS_DKP_PREFIX = "omgp_0"; -- prefix to identify RS_DKP transmissions

--
-- FUNCTIONS
--

-- DESCRIPTION
-- Broadcasts a message to other instances of RS_DKP.
-- PARAMETERS
-- eventID: the ID of an event associated with this message, as set by the
--          master or 0 if no event is associated
-- messageType: the type of message being sent (see event types constants)
-- messageData: list of other data to be sent along with the message

function omgp_broadcastMessage(eventID, messageType, messageData)
	local serializedData = "";
	if (messageData ~= nil and table.getn(messageData) >= 1) then
		-- serializedData = omgp_stringJoin(messageData, "]")
		serializedData = table.concat(messageData, "]");
	end
	
	local transmission = eventID .. "[" .. messageType .. "[" .. serializedData;
	SendAddonMessage(RS_DKP_PREFIX, transmission, "RAID");
end

-- DESCRIPTION
-- Called when a braodcasted message is received. Decodes the message and calls
-- the appropriate handler.
-- PARAMETERS
-- prefix: the message identifying prefix
-- transmission: the message
-- chanel: the group of people the message was sent to
-- sender: the player that sent the message

function omgp_receiveBroadcastedMessage(prefix, transmission, channel, sender)
	if (prefix ~= RS_DKP_PREFIX or channel ~= "RAID") then return; end
	if (sender == UnitName("player")) then return; end
	
	local eventID, messageType, messageData = omgp_unserializeTransmission(transmission);
	if (messageType == "INST") then
		omgp_remoteDKPBegin(eventID, messageData[1], messageData[2], sender);
	elseif (messageType == "INSTACK") then
		if (omgp_currentRaidID ~= nil and omgp_masterIsMe == true) then
			omgp_pluginMessage(sender .. "'s RS_DKP plug-in is now synced with yours.");
		end
	elseif (messageType == "POLL") then
		omgp_remoteActivatedPoll(eventID, messageData[1], sender);
	elseif (messageType == "POKED") then
		omgp_playerPolledRemotely(eventID, messageData[1]);
	elseif (messageType == "CREDIT") then
		omgp_addPlayerCredit(eventID, messageData[1]);
	elseif (messageType == "SBOSS") then
		if (omgp_masterIsMe == true) then
			omgp_handleBossKill(messageData[1]);
		end
	elseif (messageType == "BOSS") then
		omgp_handleBossKill(messageData[1], eventID);
	elseif (messageType == "SLOOT") then
		if (omgp_masterIsMe == true) then
			omgp_logWinningBid(messageData[1], messageData[2], omgp_makeNumber(messageData[3]));
		end
	elseif (messageType == "LOOT") then
		omgp_logWinningBid(messageData[1], messageData[2], omgp_makeNumber(messageData[3]), eventID);
	elseif (messageType == "SLAVE") then
		if (omgp_currentRaidID == nil) then return; end
		omgp_masterIsMe = false;
		omgp_logEvent("BECOME-SLAVE", {["new_master"] = sender});
		omgp_pluginMessage(sender .. " has taken DKP master.", "NOTE");
	elseif (messageType == "INFORM") then
		if (omgp_currentRaidID == nil and omgp_masterIsMe == true) then
			omgp_masterIsMe = false;
			omgp_pluginMessage("Note: " .. sender .. " is currently recording DKP for this raid.", "NOTE");
		end
	elseif (messageType == "END") then
		omgp_remoteDKPEnd(sender);
	end
end

-- DESCRIPTION
-- Changes a serialized message into its separate unserialized components,
-- consisting of an event ID, a message type, and a sequence of strings being
-- the message data.
-- PARAMETERS
-- transmission: the serialized message
-- RETURNS
-- 1. The event ID, or nil if the message isn't properly formatted
-- 2. The message type, or nil if the message isn't properly formatted
-- 3. A table of strings being the message data, or nil if the message isn't
--    properly formatted, or an empty set if there is no message data

function omgp_unserializeTransmission(transmission)
	local messageParts = omgp_stringSplit(transmission, "%[");
	local eventID = messageParts[1];
	local messageType = messageParts[2];
	
	if (eventID == nil or messageType == nil) then return; end
	
	local serializedData = messageParts[3];
	if (serializedData ~= nil) then
		local messageData = omgp_stringSplit(serializedData, "%]");
		return omgp_makeNumber(eventID), messageType, messageData;
	else
		return omgp_makeNumber(eventID), messageType, {}
	end
end

-- DESCRIPTION
-- If the player is recording DKP, forces him to the master, and broadcasts a
-- SLAVE message.

function omgp_takeMaster()
	if (omgp_currentRaidID == nil) then
		omgp_pluginMessage("You are not currently recording DKP, nor part of a recording linkup.", "ERROR");
		return;
	end
	omgp_broadcastMessage(0, "SLAVE");
	omgp_masterIsMe = true;
	omgp_logEvent("BECOME-MASTER");
	omgp_pluginMessage("You have taken DKP master.", "NOTE");
end

-- DESCRIPTION
-- If the player is recording DKP and is the master, broadcasts an INFORM
-- message.
-- PARAMETERS
-- newPlayer: the new guy who just joined the raid

function omgp_broadcastDKPInProgressInform(newPlayer)
	if (omgp_currentRaidID == nil) then return; end
	if (omgp_masterIsMe == false) then return; end
	
	omgp_broadcastMessage(0, "INFORM", {newPlayer});
end